"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var helper_plugin_utils_1 = require("@babel/helper-plugin-utils");
var helper_module_imports_1 = require("@babel/helper-module-imports");
var plugin_syntax_typescript_1 = __importDefault(require("@babel/plugin-syntax-typescript"));
var core_1 = require("@babel/core");
// import ts from 'typescript';
var addToClass_1 = __importDefault(require("./addToClass"));
var addToFunctionOrVar_1 = __importDefault(require("./addToFunctionOrVar"));
var extractTypeProperties_1 = __importDefault(require("./extractTypeProperties"));
exports.default = helper_plugin_utils_1.declare(function (api) {
    api.assertVersion(7);
    // let program;
    // let checker;
    var reactImportedName = '';
    var propTypesImportedName = 'PropTypes';
    var hasPropTypesImport = false;
    var componentCount = 0;
    var types = {};
    return {
        inherits: plugin_syntax_typescript_1.default,
        manipulateOptions: function (opts, parserOptions) {
            // Inheriting the syntax doesn't seem to define these
            parserOptions.plugins.push('classProperties', 'classPrivateProperties', 'jsx');
        },
        // pre(state: any) {
        //   program = ts.createProgram([state.opts.filename], {});
        //   checker = program.getTypeChecker();
        //   console.log(checker.getSymbolAtLocation(program.getSourceFile(state.opts.filename)!));
        // },
        visitor: {
            Program: {
                enter: function (path) {
                    // Find existing `react` and `prop-types` imports
                    path.node.body.forEach(function (node) {
                        if (!core_1.types.isImportDeclaration(node)) {
                            return;
                        }
                        if (node.source.value === 'prop-types') {
                            hasPropTypesImport = true;
                            node.specifiers.forEach(function (spec) {
                                if (core_1.types.isImportDefaultSpecifier(spec) || core_1.types.isImportNamespaceSpecifier(spec)) {
                                    propTypesImportedName = spec.local.name;
                                }
                            });
                        }
                        if (node.source.value === 'react') {
                            node.specifiers.forEach(function (spec) {
                                if (core_1.types.isImportDefaultSpecifier(spec) || core_1.types.isImportNamespaceSpecifier(spec)) {
                                    reactImportedName = spec.local.name;
                                }
                            });
                        }
                    });
                    // Add `prop-types` import if it does not exist.
                    // We need to do this without a visitor as we need to modify
                    // the AST before anything else has can run.
                    if (!hasPropTypesImport) {
                        helper_module_imports_1.addDefault(path, 'prop-types', {
                            nameHint: 'pt',
                        });
                        // Babel prefixes with an underscore
                        propTypesImportedName = "_pt";
                    }
                    path.traverse({
                        // `class Foo extends React.Component<Props> {}`
                        // @ts-ignore
                        'ClassDeclaration|ClassExpression': function (path) {
                            var node = path.node;
                            // prettier-ignore
                            var valid = node.superTypeParameters && (
                            // React.Component, React.PureComponent
                            (core_1.types.isMemberExpression(node.superClass) &&
                                core_1.types.isIdentifier(node.superClass.object, { name: reactImportedName }) && (core_1.types.isIdentifier(node.superClass.property, { name: 'Component' }) ||
                                core_1.types.isIdentifier(node.superClass.property, { name: 'PureComponent' }))) ||
                                // Component, PureComponent
                                (reactImportedName && (core_1.types.isIdentifier(node.superClass, { name: 'Component' }) ||
                                    core_1.types.isIdentifier(node.superClass, { name: 'PureComponent' }))));
                            if (valid) {
                                addToClass_1.default(node, types, { reactImportedName: reactImportedName, propTypesImportedName: propTypesImportedName });
                                componentCount += 1;
                            }
                        },
                        // `function Foo(props: Props) {}`
                        FunctionDeclaration: function (path) {
                            var node = path.node;
                            var param = node.params[0];
                            // prettier-ignore
                            var valid = param &&
                                reactImportedName &&
                                node.id.name.match(/^[A-Z]/) && (
                            // (props: Props)
                            (core_1.types.isIdentifier(param) && param.typeAnnotation) ||
                                // ({ ...props }: Props)
                                (core_1.types.isObjectPattern(param) && param.typeAnnotation));
                            if (valid) {
                                addToFunctionOrVar_1.default(path, node.id.name, types, {
                                    reactImportedName: reactImportedName,
                                    propTypesImportedName: propTypesImportedName,
                                });
                                componentCount += 1;
                            }
                        },
                        // `const Foo: React.SFC<Props> = () => {};`
                        VariableDeclaration: function (path) {
                            var node = path.node;
                            if (node.declarations.length === 0) {
                                return;
                            }
                            var id = node.declarations[0].id;
                            if (!id.typeAnnotation || !id.typeAnnotation.typeAnnotation) {
                                return;
                            }
                            var type = id.typeAnnotation.typeAnnotation;
                            // prettier-ignore
                            var valid = core_1.types.isTSTypeReference(type) &&
                                type.typeParameters &&
                                type.typeParameters.params.length > 0 && (
                            // React.SFC, React.StatelessComponent
                            (core_1.types.isTSQualifiedName(type.typeName) &&
                                core_1.types.isIdentifier(type.typeName.left, { name: reactImportedName }) &&
                                (core_1.types.isIdentifier(type.typeName.right, { name: 'SFC' }) ||
                                    core_1.types.isIdentifier(type.typeName.right, { name: 'StatelessComponent' }))) ||
                                // SFC, StatelessComponent
                                (reactImportedName && (core_1.types.isIdentifier(type.typeName, { name: 'SFC' }) ||
                                    core_1.types.isIdentifier(type.typeName, { name: 'StatelessComponent' }))));
                            if (valid) {
                                addToFunctionOrVar_1.default(path, id.name, types, {
                                    reactImportedName: reactImportedName,
                                    propTypesImportedName: propTypesImportedName,
                                });
                                componentCount += 1;
                            }
                        },
                        // `interface FooProps {}`
                        // @ts-ignore
                        TSInterfaceDeclaration: function (_a) {
                            var node = _a.node;
                            types[node.id.name] = extractTypeProperties_1.default(node, types);
                        },
                        // `type FooProps = {}`
                        TSTypeAliasDeclaration: function (_a) {
                            var node = _a.node;
                            types[node.id.name] = extractTypeProperties_1.default(node, types);
                        },
                    });
                },
                exit: function (path) {
                    if (componentCount !== 0) {
                        return;
                    }
                    // Remove the `prop-types` import of no components exist
                    path.get('body').forEach(function (bodyPath) {
                        if (core_1.types.isImportDeclaration(bodyPath.node) &&
                            bodyPath.node.source.value === 'prop-types') {
                            bodyPath.remove();
                        }
                    });
                },
            },
        },
    };
});
